"""
Overview
========

Used to login with the bot in order to make the bot join, leave, channel
and perform other irc operations.

Commands
========

Command: @login password
Description: Login with the bot.

Command: @irccmd "irc command"
Description: Make the bot send the irc command

Example
=======

<\tau> @login bot_password
<\amelia> Logged!
<\tau> @irccmd "JOIN #vy"

"""

from ameliabot.adm import is_adm
from untwisted.network import xmap
from untwisted.plugins.irc import send_cmd
from ameliabot.cmd import command

def install(server, *args):
    xmap(server, 'CMSG', irc_cmd)
    xmap(server, 'PMSG', irc_cmd)

@command('@irccmd data')
def irc_cmd(server, nick, user, host, target, msg, data):
    # is_adm(host) checks whether the user is authenticated
    # in order to send back to the server the irc command.
    if is_adm(host): send_cmd(server, data)










"""
Overview
========

Used to log msgs into a folder.

"""

from time import asctime
from untwisted.network import xmap
from os.path import join

class LogMsg(object):
    def __init__(self, server, folder):
        """
        The folder is where to save the logs.
        Example:
        LogMsg(server, '/home/tau')

        Notice it shouldn't be '/home/tau/'.
        """

        self.folder = folder
        xmap(server, 'CMSG', self.store_chan)
        xmap(server, 'PMSG', self.store_user)

    def store_chan(self, server, nick, user, host, target, msg):
        with open(join(self.folder, target), 'a+') as fd:
            fd.write('(%s)<%s> %s\n' % (asctime(), nick, msg))

    def store_user(self, server, nick, user, host, target, msg):
        with open(join(self.folder, nick), 'a+') as fd:
            fd.write('(%s)<%s> %s\n' % (asctime(), nick, msg))


install = LogMsg



"""
Overview
========

Used to do quick searches on google.

Commands
========

Command: @find ameliabot
Description: Searches on google for ameliabot.

"""

from random import choice
from gsearch import GSearch
from untwisted.plugins.irc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import command

class QuickSearch(object):
    def __init__(self, server):
        self.gsearch = GSearch()
        xmap(server, 'CMSG', self.find)
    
    @command('@find data')
    def find(self, server, nick, user, host, target, msg, data):
        result = self.gsearch.quick_search(v='0.1', q=data, safe='high')
        send_msg(server, target, str(choice(result)))
    
install = QuickSearch










""" 
Overview
========

This plugin implements a mechanism to execute code on codepad using a sophisticated approach.
It uses a scheme of tags like #py to start a sequence of code and #end to close. It supports
other langs like c, haskell etc.

Example
=======

<\tau> =py
<\tau> def calc():
<\tau> 	print 'hello world'
<\tau> calc()
<\tau> =end
<\amelia>  hello world

<\tau> =c
<\tau> int main(int argv, char *argv[]) {
<\tau> 	printf("hello world");
<\tau> 	return 0;
<\tau> }
<\tau> =end
<\amelia> : error: conflicting types for 'argv' : error: previous definition of 'argv' was here

"""

import libpad
from untwisted.plugins.irc import send_msg
from untwisted.network import xmap
from untwisted.tools import coroutine

class Codenv(object):
    def __init__(self, server, lang, start_tag, end_tag, max_width=512 * 3):
        self.lang = lang
        self.max_width = max_width
        self.start_tag = start_tag
        self.end_tag   = end_tag
        xmap(server, 'CMSG', self.process)

    @coroutine
    def process(self, server, nick, user, host, target, msg):
        if not msg == self.start_tag: return
        code = ''
        
        while True:
            args = yield server, 'CMSG'
            if args[3] == target and args[2] == host:
                if args[4] == self.end_tag:
                    break
                code = code + args[4] + '\n'
        url, output = libpad.sandbox(code, self.lang)
        if len(output) <= self.max_width:
            send_msg(server, target, output)
        else:
            send_msg(server, target, url)

install = Codenv














"""
Overview
========

Used to translate all text from an user.

Commands
========

Command: @polyglot-add lang_x lang_y ident
Description: Whenever an user with ident types it translates from lang_x to lang_y.

Command: @polyglot-del lang_x lang_y ident
Description: Remove the user with ident from the list of users to have their msgs to be translated.
"""

from untwisted.network import spawn, xmap, zmap
from libdict import GoogleTranslator
from untwisted.plugins.irc import send_msg
from ameliabot.cmd import command

source = GoogleTranslator()

def install(server):
    xmap(server, 'CMSG', split)
    xmap(server, 'CMSG', add)
    xmap(server, 'CMSG', rm)

@command('@polyglot-add lang_x lang_y ident')
def add(server, nick, user, host, target, 
               msg, lang_x, lang_y, ident):
    xmap(server, ident, listen, lang_x, lang_y)

@command('@polyglot-del lang_x lang_y ident')
def rm(server, nick, user, host, target, 
               msg, lang_x, lang_y, ident):
    zmap(server, ident, listen, lang_x, lang_y)

def listen(server, nick, user, target, msg, lang_x, lang_y):
    data = source.translate(msg, lang_x, lang_y) 
    send_msg(server, target, '%s %s' % (nick, data))

def split(server, nick, user, host, target, msg):
    spawn(server, host, nick, user, target, msg)














"""
Overview
========

Used to show players' rating on lichess.org.

Commands
========

Command: @lichess player
Description: Shows the player's rating on lichess.org.

"""

import requests
from untwisted.plugins.irc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import command

def getPlayerInfo(player):
    r = requests.get(('http://lichess.org/api/user/%s' % player))
    return r.json()

def ircOutput(playerinfo):
    if not playerinfo:
        return 'not found'
    return ("lichess.org - %s's performance: bullet: %s "
           "blitz: %s classical: %s" % 
           (playerinfo['username'], playerinfo['perfs']['bullet']['rating'],
            playerinfo['perfs']['blitz']['rating'], playerinfo['perfs']['classical']['rating']))

class Lichess:
	def __init__(self,server):
		xmap(server,'CMSG',self.getinfo)

	@command('@lichess player')
	def getinfo(self, server, nick, user, host, target, msg, player):
		result = getPlayerInfo(player)
		send_msg(server,target,str(ircOutput(result)))

install = Lichess



"""
Overview
========

Used to reconnect if the connection dies.


"""

from untwisted.network import xmap
from untwisted.iostd import CLOSE
from ameliabot.core import connect

def install(server):
    xmap(server, CLOSE, reconnect)

def reconnect(server, err):
    connect(server.servaddr, server.port, server.nick,
    server.user, server.nick_passwd, server.adm_passwd, server.chan_list, 
    server.plugmap)









"""
Overview
========

Used to download files from the bot.

Commands
========

Command: @dcc-send filename port
Description: Send file whose name is filename to the nick that issued the command.

See: booklist plugin for listing shared files.
"""

from untwisted.network import *
from untwisted.iostd import CLOSE, CONNECT_ERR
from untwisted.plugins.irc import *
from untwisted.iputils import ip_to_long
from os.path import getsize, join
from socket import error
from ameliabot.cmd import command

HEADER = '\001DCC SEND %s %s %s %s\001' 

class Send(object):
    def __init__(self, server, folder):
        self.folder = folder
        xmap(server, 'CMSG', self.dcc_send)

    @command('@dcc-send filename port')
    def dcc_send(self, server, nick, user, host, target, msg, filename, port):
    
        path = join(self.folder, filename)
        size = getsize(path)
        fd   = open(path, 'rb')
    
        try:
            dccserv = DccServer(fd, int(port))
        except error:
            send_msg(server, nick, "It couldn't listen on the port")
        else:
            request = HEADER % (filename, ip_to_long(server.myaddr), port, size)

            send_msg(server, nick, request)
        
            def is_done(dccserv, client, msg):
                send_msg(server, nick, msg)
                fd.close()
        
            xmap(dccserv, DONE, is_done, 'Done.')
            xmap(dccserv, CLOSE, lambda dccserv, client, err: is_done(dccserv, client, 'Failed.'))
            xmap(dccserv, ACCEPT_ERR, lambda dccserv, err: is_done(dccserv, None, "Accept error."))
        
            # TIMEOUT is an event that occurs in the dccsev spin
            # instance not in the client instance.
            # The client instance is the spin instance that
            # corresponds to the client socket. So, we need to pass
            # None otherwise we get an exception. The None would correspond
            # to client in the position at is_done.
            xmap(dccserv, TIMEOUT, is_done, None, "TIMEOUT. Server is down.")
        
        
install = Send










"""
"""

from untwisted.network import xmap
from random import *
from re import split
from os.path import dirname, join
from untwisted.plugins.irc import send_msg
from ameliabot.cmd import command

class Quote(object):
    def __init__(self, server):
        fd   = open(join(dirname(__file__), 'quote_database'), 'r')
        data = fd.read()
        fd.close()
        self.data = split('\n+', data)
        
        xmap(server, 'CMSG', self.send_quote)

    @command('@quote')
    def send_quote(self, server, nick, user, host, target, msg):    
        quote = choice(self.data)
        send_msg(server, target, quote)
    
install = Quote











from time import asctime
from random import choice
from untwisted.network import xmap
from untwisted.plugins.irc import send_msg

class Spam(object):
    def __init__(self, server, db=['Ameliabot rocks'], excpt=['#freenode']):
        """

        """

        self.pmed = list()
        xmap(server, 'CMSG', self.send_spam)

        # The list of chans whose users shouldnt be spammed.
        self.db    = db
        self.excpt = excpt
        
        
    def send_spam(self, server, nick, user, host, target, msg):
        if host in self.pmed or target.lower() in self.excpt: 
            return
    
        msg = choice(self.db)
        self.pmed.append(host)
        send_msg(server, nick, msg)
    
install = Spam



"""
Overview
========

Used to extract url titles from links.

"""

from untwisted.plugins.irc import send_msg
from re import search, compile, sub
from untwisted.network import xmap
import requests
from ehp import *

STR_LINK  = '(?P<address>http[s]?://[^ ]*)'
REG_LINK  = compile(STR_LINK)

class UrlTitle(object):
    def __init__(self, server):
        xmap(server, 'CMSG', self.check)
    
    def check(self, server, nick, user, host, target, msg):
        struct = search(REG_LINK, msg)
        if not struct: 
            return
        req   = requests.get(struct.group('address'))
        html  = Html()
        dom   = html.feed(req.text.encode(req.encoding))
        title = dom.fst('title').text()
        send_msg(server, target, title)

install = UrlTitle








""" 
Overview
========

Used to list files that are being shared.

Commands
========

Command: @dcc-list
Description: List all files that exist in a given folder that was shared.

"""

import libpad
from untwisted.network import xmap
from untwisted.plugins.irc import send_msg
from ameliabot.cmd import command
import os

class BookList(object):
    def __init__(self, server, folder):
        """
        """

        self.folder = folder
        xmap(server, 'CMSG', self.list_folder)
    
    @command('@dcc-list')
    def list_folder(self, server, nick, user, host, target, msg):
        """
        """
        content = '\n'.join(os.listdir(self.folder))
    
        url, _ = libpad.sandbox(content, '') 
        send_msg(server, target, url)
        









"""
Overview
========

This plugin is used to pipe channels. Whenever one types in a channel it appears in other.

Commands
========

Command: @pipe-add chan_x chan_y
Description: Pipes all text from chan_x to chan_y.

Command: @pipe-del chan_x chan_y
Description: Remove the pipe from chan_x to chan_y.

"""

from untwisted.network import xmap
from untwisted.plugins.irc import send_msg
from ameliabot.cmd import command

mapping = dict()

def install(server):
    xmap(server, 'CMSG', pipe_add)
    xmap(server, 'CMSG', pipe_rm)
    xmap(server, 'CMSG', pipe_chan)

@command('@pipe-add chan_x chan_y')
def pipe_add(server, nick, user, host, target, 
               msg, chan_x, chan_y):

    chan_x, chan_y = chan_x.upper(), chan_y.upper()
    if not mapping.has_key((server, chan_x)):
        mapping[(server, chan_x)] = list()
    mapping[(server, chan_x)].append(chan_y)

@command('@pipe-del chan_x chan_y')
def pipe_rm(server, nick, user, 
                    host, target, msg, chan_x, chan_y):
    chan_x, chan_y = chan_x.upper(), chan_y.upper()
    mapping[(server, chan_x)].remove(chan_y)

def pipe_chan(server, nick, user, host, target, msg,):
    chan_list = mapping.get((server, target.upper()))
    if not chan_list: return

    for ind in chan_list:
        send_msg(server, ind, '(%s)%s %s' % (target, nick, msg))















"""
Usage: 

# ./amelia/ameliarc

from ameliabot.plugins import antispam

def PLUGIN_SCHEME(server):
    # 1 lines per second, command to be executed if the rate is overriden.
    antispam.install(server, rate=1,  cmd='kick {chan} {nick} :Amelia rocks!')

"""

from untwisted.plugins.irc import send_cmd, send_msg
from untwisted.network import xmap
from ameliabot.cmd import regcmd
import time

class AntiSpam(object):
    def __init__(self, server, rate=1, cmd='kick {chan} {nick} :Amelia rocks!'):
        """
        Example for rate:
            rate = lines/second.

        Example for command:
            cmd = 'kick {chan} {nick} :Amelia rocks!'
            cmd = 'chanserv akick {chan} add *!*@{host} : get out!
        """
        xmap(server, 'CMSG', self.watcher)
        self.data = {}
        self.rate = rate
        self.cmd  = cmd
        self.MAX  = 2

    def watcher(self, server, nick, user, host, target, msg):
        count, start       = self.data.setdefault(host, [0,  time.time()])
        count              = count + 1
        self.data[host][0] = count

        if self.rate < count/(time.time() - start):
            send_cmd(server, self.cmd.format(nick=nick, chan=target, host=host, user=user))

        if time.time() - start > self.MAX:
            self.data[host][0], self.data[host][1] = 0, time.time()

install = AntiSpam













"""
Overview
========

Used to translate quickly some word from lang1 to lang2.

Commands
========

Command: @g lang1 lang2 data
Description: Translate the data from lang1 to lang2.
"""

from libdict import GoogleTranslator
from untwisted.network import xmap
from untwisted.plugins.irc import send_msg
from ameliabot.cmd import regcmd

class Translator(object):
    def __init__(self, server):
        self.google_translator = GoogleTranslator()
        xmap(server, 'CMSG', self.translate)

    @regcmd('@g (?P<lang1>.+?) (?P<lang2>.+?) (?P<data>.+)$')
    def translate(self, server, nick, user, host, target, msg, lang1, lang2, data):
        data = self.google_translator.translate(data, lang1, lang2)
        send_msg(server, target, data)
    
install = Translator













"""
Overview
========
This plugin uses wolfram alpha to compute mathematical expressions.

Commands
========

Command: @calc expression
Description: Computes the expression using wolfram alpha.

Example
=======

<\tau> @calc integrate x^2
<\amelia>  integral x^2  dx = x^3/3+constant

"""

import wolframalpha

from untwisted.plugins.irc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import regcmd
from ameliabot.tools import send_lines

class Calculate(object):
    def __init__(self, server, appid):
        self.client = wolframalpha.Client(appid)
        xmap(server, 'CMSG', self.calculate)
    
    @regcmd('@calc (?P<exp>.+)$')
    def calculate(self, server, nick, user, host, target, msg, exp):
        req  = self.client.query(exp)
        data = ''
        for pod in req:
            if pod.text:
                data = '%s %s' % (data, pod.text)
        send_lines(server, target, data)
    
install = Calculate











"""
Overview
========

Used to add a note to a peer when he joins the channel.

Commands
========

Command: @note-add peer data
Description: Add a msg to be shown when a nick joins a channel where the bot is in.

Command: @note-del peer
Description: Remove a note from a nick.
"""

from untwisted.network import xmap
from untwisted.plugins.irc import send_msg
from ameliabot.cmd import command

class Note(object):
    def __init__(self, server):
        xmap(server, 'CMSG', self.note_add)
        xmap(server, 'CMSG', self.note_rm)
        xmap(server, 'JOIN', self.send_note)
        self.base = {}

    @command('@note-add peer data')
    def note_add(self, server, nick, user, host, target, msg, peer, data):
        self.base[peer.lower()] = data

    @command('@note-del peer')
    def note_rm(self, server, nick, user, host, target, peer):
        del self.base[peer.lower()]

    def send_note(self, server, nick, user, host, channel):
        try:
            data = self.base[nick.lower()]
        except KeyError:
            pass
        else:
            send_msg(server, channel, '%s %s' % (nick, data))

install = Note










from untwisted.network import xmap
from untwisted.plugins.irc import send_msg
from ameliabot.cmd import command

known = set()

def install(server):
    xmap(server, 'PMSG', login)

@command('@login password')
def login(server, nick, user, host, target, msg, password):
    if password == server.adm_passwd:
        known.add(host)
        send_msg(server, nick, 'Logged!')
    else:
        send_msg(server, nick, 'Invalid password!')

def is_adm(host):
    return host in known







from untwisted.plugins.irc import send_msg
from untwisted.timer import Sched, CancelCall
from re import escape

def consume(seq):
    try:
        seq.next()
    except StopIteration:
        raise CancelCall

def send_lines(server, target, msg, delay=0.7):
    def lazy():
        for ind in msg.splitlines():
            send_msg(server, target, ind)
            yield    
    Sched(delay, consume, lazy())






"""
Overview
========

Tells the bot to tell something to someone.

Commands
========

Command: @tell person note
Description: Instructs the bot to send a note to person.

"""

from untwisted.plugins.irc import send_msg
from untwisted.network import xmap
from ameliabot.cmd import command

def install(server):
    xmap(server, 'CMSG', send)
    xmap(server, 'PMSG', send)

@command('@tell person note')
def send(server, nick, user, host, target, msg, person, note):
        send_msg(server, person, note)










